@@ -34,13 +34,13 @@ rsde1d <- function(object, ...)  UseMethod("rsde1d")
 rsde1d.default <- function(object,at,...)
                      {
     class(object) <- "snssde1d"
-    if (missing(at)) {at = object$T}
-    if (any(object$T < at || object$t0 > at) )  stop( " please use 't0 <= at <= T'")
-    if (object$M == 1){  X = matrix(object$X,nrow=length(object$X),ncol=1)}else{X = object$X}
+    if (missing(at)) {at = as.numeric(object$T)}
+    if (any(as.numeric(object$T) < at || as.numeric(object$t0) > at) )  stop( " please use 't0 <= at <= T'")
+    if (as.numeric(object$M) == 1){  X = matrix(object$X,nrow=length(object$X),ncol=1)}else{X = object$X}
     xx   <- as.vector(X[which(time(object)==at),])
     if (length(xx) == 0){
-	if (object$M==1){ F   <- lapply(1:object$M,function(i) approxfun(time(object),object$X))}else{
-               F   <- lapply(1:object$M,function(i) approxfun(time(object),object$X[,i]))}
+	if (as.numeric(object$M==1)){ F   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$X))}else{
+               F   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$X[,i]))}
                xx   <- sapply(1:length(F),function(i) F[[i]](at)) 
     }
     return(xx)
@@ -57,22 +57,22 @@ rsde2d <- function(object, ...)  UseMethod("rsde2d")
 rsde2d.default <- function(object,at,...)
                     { 
     class(object) <- "snssde2d"
-    if (missing(at)) {at = object$T}
-    if (any(object$T < at || object$t0 > at) )  stop( " please use 't0 <= at <= T'")
-    if (object$M == 1){  X = matrix(object$X,nrow=length(object$X),ncol=1)
+    if (missing(at)) {at = as.numeric(object$T)}
+    if (any(as.numeric(object$T) < at || as.numeric(object$t0) > at) )  stop( " please use 't0 <= at <= T'")
+    if (as.numeric(object$M) == 1){  X = matrix(object$X,nrow=length(object$X),ncol=1)
 	                     Y = matrix(object$Y,nrow=length(object$Y),ncol=1)}else{
 				  X = object$X
 				  Y = object$Y}
     x   <- as.vector(X[which(as.vector(time(object))==at),])
     y   <- as.vector(Y[which(time(object)==at),])
     if (length(x) == 0){
-	if (object$M==1){ Fx   <- lapply(1:object$M,function(i) approxfun(time(object),object$X))}else{
-                      Fx   <- lapply(1:object$M,function(i) approxfun(time(object),object$X[,i]))}
+	if (as.numeric(object$M)==1){ Fx   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$X))}else{
+                      Fx   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$X[,i]))}
                       x   <- sapply(1:length(Fx),function(i) Fx[[i]](at)) 
     }
     if (length(y) == 0){
-	if (object$M==1){ Fy   <- lapply(1:object$M,function(i) approxfun(time(object),object$Y))}else{
-                      Fy   <- lapply(1:object$M,function(i) approxfun(time(object),object$Y[,i]))}
+	if (as.numeric(object$M)==1){ Fy   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$Y))}else{
+                      Fy   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$Y[,i]))}
                       y   <- sapply(1:length(Fy),function(i) Fy[[i]](at)) 
     }
     return(data.frame(x,y))
@@ -89,9 +89,9 @@ rsde3d <- function(object, ...)  UseMethod("rsde3d")
 rsde3d.default <- function(object,at,...)
                     { 
     class(object) <- "snssde3d"
-    if (missing(at)) {at = object$T}
-    if (any(object$T < at || object$t0 > at) )  stop( " please use 't0 <= at <= T'")		
-    if (object$M == 1){  X = matrix(object$X,nrow=length(object$X),ncol=1)
+    if (missing(at)) {at = as.numeric(object$T)}
+    if (any(as.numeric(object$T) < at || as.numeric(object$t0) > at) )  stop( " please use 't0 <= at <= T'")		
+    if (as.numeric(object$M) == 1){  X = matrix(object$X,nrow=length(object$X),ncol=1)
 	              Y = matrix(object$Y,nrow=length(object$Y),ncol=1)
 				  Z = matrix(object$Z,nrow=length(object$Z),ncol=1)}else{
 				  X = object$X
@@ -101,18 +101,18 @@ rsde3d.default <- function(object,at,...)
     y   <- as.vector(Y[which(time(object)==at),])
     z   <- as.vector(Z[which(time(object)==at),])
     if (length(x) == 0){
-	if (object$M==1){ Fx   <- lapply(1:object$M,function(i) approxfun(time(object),object$X))}else{
-               Fx   <- lapply(1:object$M,function(i) approxfun(time(object),object$X[,i]))}
+	if (as.numeric(object$M)==1){ Fx   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$X))}else{
+               Fx   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$X[,i]))}
                x   <- sapply(1:length(Fx),function(i) Fx[[i]](at)) 
     }
     if (length(y) == 0){
-	if (object$M==1){ Fy   <- lapply(1:object$M,function(i) approxfun(time(object),object$Y))}else{
-               Fy   <- lapply(1:object$M,function(i) approxfun(time(object),object$Y[,i]))}
+	if (as.numeric(object$M)==1){ Fy   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$Y))}else{
+               Fy   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$Y[,i]))}
                y   <- sapply(1:length(Fy),function(i) Fy[[i]](at)) 
     }
     if (length(z) == 0){
-	if (object$M==1){ Fz   <- lapply(1:object$M,function(i) approxfun(time(object),object$Z))}else{
-               Fz   <- lapply(1:object$M,function(i) approxfun(time(object),object$Z[,i]))}
+	if (as.numeric(object$M)==1){ Fz   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$Z))}else{
+               Fz   <- lapply(1:as.numeric(object$M),function(i) approxfun(time(object),object$Z[,i]))}
                z   <- sapply(1:length(Fz),function(i) Fz[[i]](at)) 
     }
     return(data.frame(x,y,z))
